#include "thread/Thread.h"

#include <threads.h>

#include "data/HashMap.h"

static Core::HashMap<Core::Thread::Id, thrd_t> threads;
static Core::Thread::Id idCounter = 0;

Core::Thread::Id Core::Thread::start(Function f, void* p) {
    Id id = idCounter++;
    thrd_t* t = threads.tryEmplace(id);
    if(t == nullptr) {
        return ERROR;
    }
    if(thrd_create(t, f, p) != thrd_success) {
        (void)threads.remove(id);
        return CORE_ERROR(Core::Error::THREAD_ERROR);
    }
    return id;
}

bool Core::Thread::join(Id id, int* returnValue) {
    thrd_t* t = threads.search(id);
    if(t == nullptr) {
        return CORE_ERROR(Core::Error::INVALID_ID);
    }
    if(thrd_join(*t, returnValue) != thrd_success) {
        return CORE_ERROR(Core::Error::THREAD_ERROR);
    }
    return threads.remove(id);
}